/* ***********************************************************
 * This file was automatically generated on 2012-09-28.      *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generator git on tinkerforge.com                   *
 *************************************************************/

#ifndef BRICKLET_BAROMETER_H
#define BRICKLET_BAROMETER_H

#include "ip_connection.h"

/**
 * \defgroup BrickletBarometer Barometer Bricklet
 */

/**
 * \ingroup BrickletBarometer
 *
 * Device for sensing air pressure and altitude changes
 */
typedef Device Barometer;

/**
 * \ingroup BrickletBarometer
 *
 * This callback is triggered periodically with the period that is set by
 * {@link barometer_set_air_pressure_callback_period}. The parameter is the air pressure of the
 * air pressure sensor.
 * 
 * {@link BAROMETER_CALLBACK_AIR_PRESSURE} is only triggered if the air pressure has changed since the
 * last triggering.
 */
#define BAROMETER_CALLBACK_AIR_PRESSURE 15

/**
 * \ingroup BrickletBarometer
 *
 * This callback is triggered periodically with the period that is set by
 * {@link barometer_set_altitude_callback_period}. The parameter is the altitude of the
 * air pressure sensor.
 * 
 * {@link BAROMETER_CALLBACK_ALTITUDE} is only triggered if the altitude has changed since the
 * last triggering.
 */
#define BAROMETER_CALLBACK_ALTITUDE 16

/**
 * \ingroup BrickletBarometer
 *
 * This callback is triggered when the threshold as set by
 * {@link barometer_set_air_pressure_callback_threshold} is reached.
 * The parameter is the air pressure of the air pressure sensor.
 * 
 * If the threshold keeps being reached, the callback is triggered periodically
 * with the period as set by {@link barometer_set_debounce_period}.
 */
#define BAROMETER_CALLBACK_AIR_PRESSURE_REACHED 17

/**
 * \ingroup BrickletBarometer
 *
 * This callback is triggered when the threshold as set by
 * {@link barometer_set_altitude_callback_threshold} is reached.
 * The parameter is the altitude of the air pressure sensor.
 * 
 * If the threshold keeps being reached, the callback is triggered periodically
 * with the period as set by {@link barometer_set_debounce_period}.
 */
#define BAROMETER_CALLBACK_ALTITUDE_REACHED 18

/**
 * \ingroup BrickletBarometer
 *
 * Creates an object with the unique device ID \c uid. This object can then be
 * added to the IP connection.
 */
void barometer_create(Barometer *barometer, const char *uid);

/**
 * \ingroup BrickletBarometer
 *
 * Returns the air pressure of the air pressure sensor. The value
 * has a range of 10000 to 1200000 and is given in mbar/1000, i.e. a value
 * of 1001092 means that an air pressure of 1001.092 mbar is measured.
 * 
 * If you want to get the air pressure periodically, it is recommended to use the
 * callback {@link BAROMETER_CALLBACK_AIR_PRESSURE} and set the period with
 * {@link barometer_set_air_pressure_callback_period}.
 */
int barometer_get_air_pressure(Barometer *barometer, int32_t *ret_air_pressure);

/**
 * \ingroup BrickletBarometer
 *
 * Returns the relative altitude of the air pressure sensor. The value is given in
 * cm and represents the difference between the current altitude and the reference
 * altitude that can be set with {@link barometer_calibrate_altitude}.
 * 
 * If you want to get the altitude periodically, it is recommended to use the
 * callback {@link BAROMETER_CALLBACK_ALTITUDE} and set the period with
 * {@link barometer_set_altitude_callback_period}.
 */
int barometer_get_altitude(Barometer *barometer, int32_t *ret_altitude);

/**
 * \ingroup BrickletBarometer
 *
 * Sets the period in ms with which the {@link BAROMETER_CALLBACK_AIR_PRESSURE} callback is triggered
 * periodically. A value of 0 turns the callback off.
 * 
 * {@link BAROMETER_CALLBACK_AIR_PRESSURE} is only triggered if the air pressure has changed since the
 * last triggering.
 * 
 * The default value is 0.
 */
int barometer_set_air_pressure_callback_period(Barometer *barometer, uint32_t period);

/**
 * \ingroup BrickletBarometer
 *
 * Returns the period as set by {@link barometer_set_air_pressure_callback_period}.
 */
int barometer_get_air_pressure_callback_period(Barometer *barometer, uint32_t *ret_period);

/**
 * \ingroup BrickletBarometer
 *
 * Sets the period in ms with which the {@link BAROMETER_CALLBACK_ALTITUDE} callback is triggered
 * periodically. A value of 0 turns the callback off.
 * 
 * {@link BAROMETER_CALLBACK_ALTITUDE} is only triggered if the altitude has changed since the
 * last triggering.
 * 
 * The default value is 0.
 */
int barometer_set_altitude_callback_period(Barometer *barometer, uint32_t period);

/**
 * \ingroup BrickletBarometer
 *
 * Returns the period as set by {@link barometer_set_altitude_callback_period}.
 */
int barometer_get_altitude_callback_period(Barometer *barometer, uint32_t *ret_period);

/**
 * \ingroup BrickletBarometer
 *
 * Sets the thresholds for the {@link BAROMETER_CALLBACK_AIR_PRESSURE_REACHED} callback.
 * 
 * The following options are possible:
 * 
 * \verbatim
 *  "Option", "Description"
 * 
 *  "'x'",    "Callback is turned off"
 *  "'o'",    "Callback is triggered when the air pressure is *outside* the min and max values"
 *  "'i'",    "Callback is triggered when the air pressure is *inside* the min and max values"
 *  "'<'",    "Callback is triggered when the air pressure is smaller than the min value (max is ignored)"
 *  "'>'",    "Callback is triggered when the air pressure is greater than the min value (max is ignored)"
 * \endverbatim
 * 
 * The default value is ('x', 0, 0).
 */
int barometer_set_air_pressure_callback_threshold(Barometer *barometer, char option, int32_t min, int32_t max);

/**
 * \ingroup BrickletBarometer
 *
 * Returns the threshold as set by {@link barometer_set_air_pressure_callback_threshold}.
 */
int barometer_get_air_pressure_callback_threshold(Barometer *barometer, char *ret_option, int32_t *ret_min, int32_t *ret_max);

/**
 * \ingroup BrickletBarometer
 *
 * Sets the thresholds for the {@link BAROMETER_CALLBACK_ALTITUDE_REACHED} callback.
 * 
 * The following options are possible:
 * 
 * \verbatim
 *  "Option", "Description"
 * 
 *  "'x'",    "Callback is turned off"
 *  "'o'",    "Callback is triggered when the altitude is *outside* the min and max values"
 *  "'i'",    "Callback is triggered when the altitude is *inside* the min and max values"
 *  "'<'",    "Callback is triggered when the altitude is smaller than the min value (max is ignored)"
 *  "'>'",    "Callback is triggered when the altitude is greater than the min value (max is ignored)"
 * \endverbatim
 * 
 * The default value is ('x', 0, 0).
 */
int barometer_set_altitude_callback_threshold(Barometer *barometer, char option, int32_t min, int32_t max);

/**
 * \ingroup BrickletBarometer
 *
 * Returns the threshold as set by {@link barometer_set_altitude_callback_threshold}.
 */
int barometer_get_altitude_callback_threshold(Barometer *barometer, char *ret_option, int32_t *ret_min, int32_t *ret_max);

/**
 * \ingroup BrickletBarometer
 *
 * Sets the period in ms with which the threshold callbacks
 * 
 *  {@link BAROMETER_CALLBACK_AIR_PRESSURE_REACHED}, {@link BAROMETER_CALLBACK_ALTITUDE_REACHED}
 * 
 * are triggered, if the thresholds
 * 
 *  {@link barometer_set_air_pressure_callback_threshold}, {@link barometer_set_altitude_callback_threshold}
 * 
 * keep being reached.
 * 
 * The default value is 100.
 */
int barometer_set_debounce_period(Barometer *barometer, uint32_t debounce);

/**
 * \ingroup BrickletBarometer
 *
 * Returns the debounce period as set by {@link barometer_set_debounce_period}.
 */
int barometer_get_debounce_period(Barometer *barometer, uint32_t *ret_debounce);

/**
 * \ingroup BrickletBarometer
 *
 * Calibrates the altitude by setting the reference altitude to the current
 * altitude.
 */
int barometer_calibrate_altitude(Barometer *barometer);

/**
 * \ingroup BrickletBarometer
 *
 * Returns the temperature of the air pressure sensor. The value
 * has a range of -4000 to 8500 and is given in °C/100, i.e. a value
 * of 2007 means that a temperature of 20.07 °C is measured.
 * 
 * This temperature is used internally for temperature compensation of the air
 * pressure measurement. It is not as accurate as the temperature measured by the
 * :ref:`temperature_bricklet` or the :ref:`temperature_ir_bricklet`.
 */
int barometer_get_chip_temperature(Barometer *barometer, int16_t *ret_temperature);

/**
 * \ingroup BrickletBarometer
 *
 * Returns the name (including the hardware version), the firmware version
 * and the binding version of the device. The firmware and binding versions are
 * given in arrays of size 3 with the syntax [major, minor, revision].
 */
int barometer_get_version(Barometer *barometer, char ret_name[40], uint8_t ret_firmware_version[3], uint8_t ret_binding_version[3]);

/**
 * \ingroup BrickletBarometer
 *
 * Registers a callback with ID \c id to the function \c callback.
 */
void barometer_register_callback(Barometer *barometer, uint8_t id, void *callback);

#endif
